# -*- coding: utf-8 -*-
# Créé par Fred, le 21/07/2014
"""
Ce module n'a pas de prétention autre que de simplifier la construction de
dessins avec des termes en français. Si vous souhaitez davantage de possibilités
vous pouvez rajouter des options ou d'autres fonctions.
"""
# http://tkinter.fdex.eu/index.html

from __future__ import division
import os
from PIL import Image, ImageTk
import Tkinter as Tk
import easyguifr as eg
from random import randrange,random
import time
import math

convdegrad=math.pi/180  #conversion des degrés en radian pour utiliser les fonctions sin et cos
convraddeg=180/math.pi

try:
    from PIL import Image   as PILImage
    from PIL import ImageTk as PILImageTk
    PILisLoaded = True
except:
    PILisLoaded = False

colors=['white','black','grey','blue','cyan','green','yellow','red','pink'
            ,'orange','magenta','brown','violet','maroon','purple','ivory','SkyBlue1','light grey']

couleursfr={'blanc':'white','noir':'black','gris':'grey','bleu':'blue','cyan':'cyan','vert':'green'
            ,'jaune':'yellow','rouge':'red','rose':'pink'
            ,'orange':'orange','magenta':'magenta','brun':'brown','violet':'violet'
            ,'marron':'maroon','pourpre':'purple','ivoire':'ivory','BleuCiel':'SkyBlue1','GrisClair':'light grey'}

Nb_couleursfr=len(couleursfr)

def couleurs(coul):
    """
    Traduction des couleurs de base : dans les fonctions ci-dessous
    on donnera les couleurs en anglais, en français ou sous la forme #rrggbb.
    On peut rajouter des couleurs dans les deux ensembles : colors et couleursfr.
    Pour les conversions, utiliser couleurs(nomCouleurFrancais)
    Voir rgb.txt pour les noms anglais avec les correspondances numériques.
    """
    if coul=='':return ''
    elif coul in colors or coul[0]=='#':return coul
    else: return couleursfr[coul]

def c_hex(x):
    try:
        a=hex(int(x%256))
    except:
        a='00'
    l=len(a)
    if a[l-2]=='x':u='0'
    else: u=a[l-2]
    return u+a[l-1]

def couleurRGB(r,g,b):
    return '#'+c_hex(r)+c_hex(g)+c_hex(b)
    """convertir un triplet rgb en valeur hexadécimale"""

def cadre_dessin_base(  fenetre=None
                        , large=500
                        , haut=500
                        , fond='ivoire'
                        , nombre_col=10
                        , nombre_ligne=10
                        , Okbouton='Quitter'):
    """
    Dessine un canevas dans lequel on peut dessiner.
    Un seul bouton est prévu : nombre_col et nombre_ligne sont utilisés pour les positionnements
    dans la fenêtre principale. Il est très simple de rajouter des boutons.
    La sortie donne les identifiants du canvas et du bouton.
    """
    if fenetre==None:
        return None, None
    else:
        u = Tk.Canvas(fenetre,width=large,height=haut,bg=couleurs(fond))
        u.grid(column=0, columnspan=nombre_col, row=0, rowspan=nombre_ligne)
        v = Tk.Button(fenetre, text=Okbouton, command=fenetre.destroy)
        v.grid(column=nombre_col,row=nombre_ligne)
        return u, v

def cree_rectangle(cadre=None,x0=0,y0=0,x1=0,y1=0,fond='blanc',trait='noir',nom=None):
    """
    Dessine un rectangle entre les points (x0,y0) et (x1,y1); il faut donner un nom!
    """
    if cadre!=None:
        cadre.create_rectangle(x0,y0,x1,y1,fill=couleurs(fond),outline=couleurs(trait),tags=nom)
    else:
        return None

def centre_rectangle(cadre,nom):
    """
    Renvoie les coordonnées du centre d'un rectangle.
    """
    if cadre!=None:
        a=cadre.coords(nom)
        return (a[0]+a[2])//2, (a[1]+a[3])//2
    else:
        return 0,0

def cree_grille(cadre=None,ncolonnes=20,nlignes=20,affiche=True,trait='noir',couleur='blanc'):
    """
    Dessine une grille de ncolonnes sur nlignes;
    Chaque case a un nom : 'numcol/numlig' qui permet de faire des modifications facilement

    """
    h=cadre.winfo_reqheight()
    l=cadre.winfo_reqwidth()
    u=h//nlignes
    v=l//ncolonnes
    if affiche: sta='normal'
    else: sta='hidden'
    for k in range(ncolonnes):
        for j in range(nlignes):
            cadre.create_rectangle(k*v,j*u,(k+1)*v,(j+1)*u
                       ,state=sta
                       ,outline=couleurs(trait)
                       ,fill=couleurs(couleur)
                       ,tags=str(k)+'/'+str(j))

def cree_arc_cercle(cadre=None,x0=0,y0=0,x1=0,y1=0,debutangle=0,angle=0,fond='blanc',trait='noir',nom=None):
    """
    Dessine un arc de cercle dans un rectangle entre les points (x0,y0) et (x1,y1);
    Angles en degrés.
    il faut donner un nom!
    """
    if cadre!=None:
        cadre.create_arc(x0,y0,x1,y1,start=debutangle,extent=angle,fill=couleurs(fond),outline=couleurs(trait),tags=nom)
    else:
        return None

def cree_cercle(cadre=None,x_C=0,y_C=0,rayon=10,fond='blanc',trait='noir',nom=None):
    if cadre!=None:
        cadre.create_oval(x_C-rayon,y_C-rayon,x_C+rayon,y_C+rayon,fill=couleurs(fond),outline=couleurs(trait),tags=nom)
    else:
        return None

def cree_ligne(cadre=None,x0=0,y0=0,x1=0,y1=0,trait='noir',epaisseur=1,nom=None):
    """
    Dessine un segment entre les points (x0,y0) et (x1,y1);
    il faut donner un nom!
    """
    if cadre!=None:
        cadre.create_line(x0,y0,x1,y1,width=epaisseur,fill=couleurs(trait),tags=nom)
    else:
        return None

def cree_texte(cadre=None,x0=0,y0=0,texte='',police=('Helvetica', '16'),trait='noir',nom=None):
    """
    Dessine un texte centré au point (x0,y0) ;
    il faut donner un nom!
    """
    if cadre!=None:
        cadre.create_text(x0,y0,font=police,text=texte,fill=couleurs(trait),tags=nom)
    else:
        return None

def cree_image(cadre=None,nomimage='',x0=0,y0=0,trait='',nom=None):
    """
    Charge une image et la place centrée au point (x0,y0) ;
    il faut donner un nom à l'objet!
    """
    msg=''
    if cadre!=None:
        tk_Image = None
        if nomimage:
            imageFilename = os.path.normpath(nomimage)
            junk,ext = os.path.splitext(imageFilename)

            if os.path.exists(imageFilename):
                if ext.lower() in [".gif", ".pgm", ".ppm"]:
                    tk_Image = Tk.PhotoImage(file=imageFilename)
                else:
                    if PILisLoaded:
                        try:
                            pil_Image = PILImage.open(imageFilename)
                            tk_Image = PILImageTk.PhotoImage(pil_Image)
                        except:
                            msg += eg.msgErreurImage % (imageFilename,
                                "\nPython Imaging Library (PIL) ne peut pas convertir ce fichier en une image affichable."
                                "\n\nPIL dit :\n" + formatage_message_erreur())

                    else:  # PIL is not loaded
                        msg += eg.msgErreurImage % (imageFilename,
                        "\n Impossible d'importer Python Imaging Library (PIL) pour afficher cette image.\n\n"
                        "Il faut installer PIL\n"
                        "(http://www.pythonware.com/products/pil/)\n"
                        "pour afficher " + ext + " fichiers d\'image.")

            else:
                msg += eg.msgErreurImage % (imageFilename, "\nFichier image pas trouvé.")

        if tk_Image:
            cadre.create_image(x0,y0,image=tk_Image,tags=nom)
            cadre.image=tk_Image
    else:
        return None

def cree_etoile(cadre=None,xC=0,yC=0,rayon=50,angle_depart=0
                ,nombre_sommets=5,fond='blanc',trait='noir',epaisseur=1,plein=True,nom=None):
    """
    Dessine une étoile au centre xC, yC, dans un cercle de rayon rayon
    Le dessin se fait sur la base des sommets d'un polygone régulier :
    l'orientation du premier segment est donnée par angle_depart (en degrés)
    """
    if nombre_sommets>0 and cadre!=None:
        angle=2*math.pi/nombre_sommets
        ag=angle_depart*convdegrad
        L=[]
        colorf=couleurs(fond)
        colort=couleurs(trait)
        for k in range(nombre_sommets):
            x=xC+rayon*math.cos(ag)
            y=yC+rayon*math.sin(ag)
            ag=ag+angle
            L.append([int(x),int(y)])
        if plein:z=1;fois=1
        else:
            if nombre_sommets%2==1:z=(nombre_sommets-1)//2;fois=1
            else:z=nombre_sommets//2-1;fois=2
        zz=[]
        for k in range(nombre_sommets):
            for w in range(fois):
                u=(k+w) % nombre_sommets
                v=(k+w+z) % nombre_sommets
                zz=zz+[L[u][0],L[u][1],L[v][0],L[v][1]]
        cadre.create_polygon(zz,fill=colorf,outline=colort,width=epaisseur,tags=nom)

def coordonnees_sur_droite(xA,yA,xB,yB,t):
    """ retourne les coordonnées d'un point sur une droite
    passant par deux points A et B. t est un nombre décimal.
    Pour que le point soit entre A et B, il faut que t soit compris entre 0 et 1.
    """
    return xA+t*(xB-xA),yA+t*(yB-yA)

def coordonnees_sur_ovale(xA,yA,xB,yB,t):
    """ retourne les coordonnées d'un point sur un ovale délimité
    par deux points A et B. t est un angle en degrés.
    On peut modifier la fonction en ajoutant un angle de depart t0
    """
    a=abs(xB-xA)/2
    b=abs(yB-yA)/2
    x=(xA+xB)/2;y=(yA+yB)/2
    z=convdegrad*t
    # zdepart=convdegrad*t0; z=z+zdepart
    return x+a*math.cos(-z),y+b*math.sin(-z)

def coordonnees_rotation(xM,yM,x_C, y_C,angle_deg):
    angle_rad=convdegrad*angle_deg
    u=math.cos(angle_rad);v=math.sin(angle_rad)
    return x_C+u*(xM-x_C)-v*(yM-y_C), y_C+v*(xM-x_C)+u*(yM-y_C)

def coordonnees_rotation_ovale(xM,yM,x_C,y_C,a,b,angle_deg):
    angle_rad=convdegrad*angle_deg
    u=b*math.cos(angle_rad);v=a*math.sin(angle_rad)
    return x_C+u*(xM-x_C)-v*(yM-y_C), y_C+v*(xM-x_C)+u*(yM-y_C)
#rayon=100, angle_depart=0, angle

def parametres_parabole(xA, yA, xB, yB, xC):
    if xA==xB or xA==xC or xB==xC:
        return 0,0,0,0,0
    else:
        u,v=xC-xA, xA
        a=(yA-yB)/((xA-xB)**2);b=-2*a*xB;c=yB+a*xB*xB
        return u,v, a,b,c

def coordonnees_sur_parabole_simple(u,v,a,b,c,t):
        x=u*t+v
        y=a*x*x+b*x+c
        return x,y

def coordonnees_sur_parabole(xA,yA,xB,yB,xC,t):
    """ retourne les coordonnées d'un point sur une parabole
    commnçant en A, de sommet B et finissant en C (on a juste besoin de l'abscisse de C).
    t est un nombre décimal compris entre 0 et 1.
    Pour une parabole orientée 'normalement' inverser y à la sortie pr rapport à
    la hauteur du canevas.
    """
    if xA==xB or xA==xC:
        return xA,yA
    elif xB==xC:
        return xC,yC
    else:
        x=(xC-xA)*t+xA
        a=(yA-yB)/((xA-xB)**2);b=-2*a*xB;c=yB+a*xB*xB
        y=a*x*x+b*x+c
        return x,y

def distance(A,B):
    """
    Calcul de la distance entre les deux listes A=[a,b] et B=[c,d]
    """
    return ((A[0]-B[0])**2+(A[1]-B[1])**2)**0.5

def calcule_ecart(a,longueur):
    """
    Renvoie la deuxième coordonnée d'un vecteur (a,b) ou (b,a) dont
    on connait la longueur.
    """
    if longueur>a:
        u=(longueur*longueur-a*a)**0.5
    else: u=0
    return u

def signe(a):
    if a>0:return 1
    elif a<0: return -1
    else: return 0

def rand1():    #renvoie +1 ou -1 aléatoirement
    # u=random.random()
    u=random()
    if u>0.5:u=1
    else: u=-1
    return u



#--------------------------------------------------------------------
###                            Démonstration                      ###
#--------------------------------------------------------------------
def bouge():
    """
    fonction utilisée pour la démonstration de l'utilisation des fonctions précédentes
    ne pas utiliser en dehors de ce module.
    """
    global root,can,dx,dy,nomI,nomR,nomC,nomL,nomT,large,haut,compteur, tailleC
    dx=randrange(-20,20);dy=randrange(-20,20)

# déplacement aléatoire du rectangle : les coordonnées sont tirées aléatoirement
    v=can.coords(nomR)
    v[0]=v[0]+dx;
    if v[0]>=large:v[0]=large-20
    if v[0]<=0:v[0]=0
    v[2]=v[0]+20
    v[1]=v[1]+dy;
    if v[1]>=haut:v[1]=haut-20
    if v[1]<=0:v[1]=0
    v[3]=v[1]+20
    can.coords(nomR,v[0],v[1],v[2],v[3])    #nouvelles coordonnées de nomR

# déplacement de l'image sur un cercle ou une parabole
    v=can.coords(nomI)      #récupération des anciennes coordonnées de nomI
    x,y=coordonnees_sur_parabole(30,30,large/2,haut/2,large-50,0.05*(compteur %20))
    #x,y=coordonnees_sur_ovale(v[0]-30,v[1]-30,v[0]+30,v[1]+30,compteur %360)
    can.coords(nomI,x,haut-y)    #nouvelles coordonnées de nomI

# déplacement des droites dont les extrémités se déplacent sur les bords horizontauxc
    v=can.coords(nomL)      #récupération des anciennes coordonnées
    v[0]=(v[0]+2) % large   #calcul des extrémités des droites
    v[2]=(v[2]-2) % large
    x,y=coordonnees_sur_droite(v[0],v[1],v[2],v[3],0.01*(compteur %100))  #calcul des coordonnées du centre de l'arc de cercle
    can.coords(nomL,v[0],v[1],v[2],v[3])    #nouvelles coordonnées de la droite
    can.itemconfig(nomL,fill=colors[randrange(0,Nb_couleursfr)])  #changement de couleur

# déplacement de l'arc de cercle sur le segment précédent
    angdepart=float(can.itemcget(nomC, 'start'))+10 % 360       #angle de rotation de la portion de camembert
    can.coords(nomC,x-tailleC/2,y-tailleC/2,x+tailleC/2,y+tailleC/2)    #nouvelles coordonnées de l'arc de cercle
    can.itemconfig(nomC,start=angdepart,fill=colors[randrange(0,Nb_couleursfr)])

# déplacement du texte au hasard
    if compteur % 8 ==0:
        v=can.coords(nomT)
        v[0]=randrange(20,large-20)
        v[1]=randrange(20,haut-20)
        can.coords(nomT,v[0],v[1])

# boucle principale : on rappelle bouge sans paramètre
    compteur+=1
    root.update
    root.after(80, bouge)

if __name__ == '__main__':
    """
    Démonstration de l'utilisation des fonctions du module.
    """
    compteur=0
    nomR ='R0'
    nomC='C0';tailleC=100
    nomL='L0'
    nomI='I0'
    nomT='T0'
    root = Tk.Tk()
    large,haut=500,400
    can,bou1=cadre_dessin_base(fenetre=root,large=large,haut=haut)
    cree_image(cadre=can,nomimage='snake2.GIF',x0=large/2,y0=haut/2,trait=None,nom=nomI)
    cree_texte(cadre=can,x0=3*large/4,y0=3*haut/4,texte='Python !',police=('Helvetica', '16'),trait='bleu',nom=nomT)
    cree_rectangle(cadre=can,x0=large/2-10,y0=haut/2-10,x1=large/2+10,y1=haut/2+10,fond='rose',trait='noir',nom=nomR)
    cree_arc_cercle(cadre=can,x0=large/4-tailleC,y0=haut/4-tailleC,x1=large/4+tailleC,y1=haut/4+tailleC,debutangle=0,angle=50
                ,fond='bleu',trait='orange',nom=nomC)
    cree_ligne(cadre=can,x0=0,y0=0,x1=large,y1=haut,trait='noir',epaisseur=1,nom=nomL)
    cree_cercle(cadre=can,x_C=large/4,y_C=3*haut/4,rayon=100,fond='vert',trait='orange')
    cree_etoile(cadre=can,xC=large/4,yC=3*haut//4,rayon=100, angle_depart=0,nombre_sommets=17,fond='orange',trait='bleu',epaisseur=1,plein=False,nom='etoile')
    bouge()
    root.mainloop()